# Softweb Solutions Inc
## IOT Connect SDK : Software Development Kit 2.2.0

Prerequisite tools:
	* Arduino IDE 1.8.5 or higher 
	* ESP board installed
		Open Arduino IDE and click on File > Preferences. In “Additional Boards Manager URLs” add below line and click on “OK”:
			ESP32 device package link “https://dl.espressif.com/dl/package_esp32_index.json”
			ESP8266 device package link “http://arduino.esp8266.com/stable/package_esp8266com_index.json”

	* Now go to Tools > Board > Boards Manager
		Enter “ESP32” and click on Install button “esp32 by Espressif System version 1.0.4 ” or higher version
		Enter “ESP8266” and click on install button “esp8266 by ESP8266 community version 2.4.2” or higher version

	* Selection of board
		ESP32 : Tools > Board > DOIT ESP32 DEVKITV1 (or else which you are using )
		ESP8266 : Tools > Board > Nodemcu 1.0 (or else which you are using )

	* Libraries installation : 
		You can open Sketch > include library > manage libraries.. in Arduino IDE. When you install ESP board in IDE mortally all required libraries will install as per device. You need to install ArduinoJson version(6.13.0)

Installation:
	* Download and Extract the “iotconnect-C-sdk-ESP-2.2.0.zip” file ffromom IoTConneect portal
		Using Arduino IDE goto respective ESP board folder.
			- iotconnect-C-sdk-ESP-2.2.0/ESP_SDK/ESP32SDK/ESP32
			- iotconnect-C-sdk-ESP-2.2.0/ESP_SDK/ESP8266SDK/ESP8266

	* Configuration guide documentation for Arduino and ESP boards. On above path you can find the reference document which is as below:
		ESP32-With-the-Arduino-IDE.doc
		ESP8266-With-the-Arduino-IDE.doc
		
	* You can take the firmware file from iotconnect-C-sdk-ESP-2.2.0/ESP_SDK/ESP32SDK/ESP32 file path
			- ESP32 : ESP32.ino
			- ESP866 : ESP8266.ino

	* Add prerequisite input data in above file as explained in the usage section as below. Also enter proper attribute/sensor name which has associated to the device.

	* Ready to go with following step.
		Using Arduino IDE click on “Upload” button or else press “ctrl + u”



Import library
```C-SDK
#include "IoTConnect.h"
```

Prerequisite input data *
```C-SDK

//"Get your ENV and CPID from the portal key vaults module or visit https://help.iotconnect.io SDK section."

   String ENV       = "yourEnvironment";
   String cpId      = "yourcpId";         // your CPID (Company ID)               
   String uniqueId  = "yourUniqueId";       // your UniqueID (Device name )
   char *ssid        = "yourNetwork";               // your network SSID (name)
   char *pass        = "secretPassword";            // your network password (use for WPA, or use as key for WEP)                                         
```  

To get the device information and connect to the device
```C-SDK
SDK.InitializationSDK(cpId, uniqueId, CallbackMessage, TwinCallbackMessage, ENV);
```

To receive the All twins from Cloud to Device(C2D)
```C-SDK
SDK.GetAllTwins();
```

Data input format
```C-SDK
DynamicJsonDocument  Attribute_json(2048);
  JsonArray Device_data = Attribute_json.to<JsonArray>();
  JsonObject Device_01 = Device_data.createNestedObject();
      Device_01["uniqueId"] = uniqueId; 
      Device_01["time"] = GetTime();
      JsonObject D01_Sensors=Device_01.createNestedObject("data");
        D01_Sensors["Humidity"] = random(20, 80);
        D01_Sensors["Temperature"] = random(12, 32); 
        JsonObject D01_SensorsV = D01_Sensors.createNestedObject("vibration");  
            D01_SensorsV["x"] = random(12,  50);  
            D01_SensorsV["y"] = random(-20, 30);  
            D01_SensorsV["z"] = random(10,  35); 
```

To send the data from Device To Cloud(D2C)
```C-SDK
SDK.SendData(Attribute_json_Data); 
```

To receive the command from Cloud to Device(C2D) with ACK to cloud
```C-SDK
void CallBack(String &topic, String &payload) {        
    String cmd_ackID="";   String Cmd_value="";
    int Status = 0,mt=0;
    Serial.println("Cmd_msg >>  " + payload);    
```

To receive the twin from Cloud to Device(C2D)
```C-SDK
String key = "firmware_version";
String value = "4.0";
void TwinCallBack(String &topic, String &payload) {          
    Serial.println("Twin_msg >>  " +topic + " " + payload);       
    //TO update the twinproperty use UpdateTwin function                             
    //SDK.UpdateTwin(key,value);
}
```
To Stoped the IoTConnect SDK
```C-SDK
// After this SDK will Stop you need to reboot your device or SDK
   SDK.Dispose();
      
   
```


## Release Note :

** New Feature **
 Update all public methods name in Pascal case

** Improvements **
1. We have updated below methods name:
    - Old : SendAck(data, time, msgType)
    - New : SendAck(data, msgType)
etc...